from __future__ import annotations
from dataclasses import dataclass
from .models.seasoned_price import SeasonedPrice
from .models.tiered_price import TieredPrice
from ..enums.price_type import PriceType


@dataclass
class LocalePrice:
    price_type: PriceType
    value: float | list[TieredPrice] | list[SeasonedPrice]
    currency:str

    def __post_init__(self):
        match self.price_type:
            case PriceType.Static:
                assert isinstance(self.value, (float,int))
            case PriceType.Seasoned:
                assert type(self.value) == list
                assert isinstance(self.value[0], SeasonedPrice)
            case PriceType.Tiered:
                assert type(self.value) == list
                assert isinstance(self.value[0], TieredPrice)
        assert len(self.currency)

    @property
    def price(self) -> float:
        match self.price_type:
            case PriceType.Static:
                return self.value
            case PriceType.Seasoned:
                return self._get_price_seasoned()
            case PriceType.Tiered:
                return self._get_price_tiered()

    def is_equal(self, other_currency: str) -> bool:
        """Use this method if necessary to test against the el-price currency"""
        return self.currency.lower == other_currency.lower

    def _get_price_seasoned(self) -> float:
        s: SeasonedPrice
        for s in self.value:
            if s.validity.valid():
                return s.value        

    def _get_price_tiered(self):
        t: TieredPrice
        for t in self.value:
            pass
    





import logging
from .hubmember import HubMember

_LOGGER = logging.getLogger(__name__)


class ChargerSwitch(HubMember):
    def __init__(
            self,
            hass,
            data_type: type,
            listenerentity,
            initval,
            currentname: str,
            ampmeter_is_attribute: bool,
            hubdata=None,
            init_override:bool = False
    ):
        self._hubdata = hubdata
        self._hass = hass
        self._value = initval
        self._current = None
        self._current_attr_name = currentname
        self._ampmeter_is_attribute = ampmeter_is_attribute
        super().__init__(data_type=data_type, listenerentity=listenerentity, init_override=init_override, initval=initval)

    @property
    def is_initialized(self) -> bool:
        if self._hubdata.chargerobject is not None and not self._hubdata.chargerobject.is_initialized:
            return False
        return super().is_initialized

    @property
    def current(self) -> int:
        if self._current is None:
            return 0
        return self._current

    @current.setter
    def current(self, value):
        try:
            self._current = int(value)
        except ValueError:
            pass

    def updatecurrent(self):
        if self._hubdata.chargertype.options.charger_is_outlet is True:
            return
        try:
            self.current = self.get_sensor_from_hass(self._current_attr_name)
        except:
            pass

    async def async_updatecurrent(self):
        if self._hubdata.chargertype.options.charger_is_outlet is True:
            return
        try:
            self.current = await self.async_get_sensor_from_hass(self._current_attr_name)
        except:
           await self.async_log_warning_once()

        # elif self._ampmeter_is_attribute is True:
        #     ret = self._hass.states.get(self.entity)
        #     if ret is not None:
        #         ret_attr = str(ret.attributes.get(self._current_attr_name))
        #         self.current = ret_attr
        #     else:
        #         self._log_warning_once()
        # else:
        #     ret = self._hass.states.get(self._current_attr_name)
        #     if ret is not None:
        #         self.current = ret.state
        #     else:
        #         self._log_warning_once()

    HASLOGGED_INITWARN = False
    async def async_log_warning_once(self):
        if not self.HASLOGGED_INITWARN:
            _LOGGER.warning("Chargerobject state was None while getting current. "
                "Updatecurrent will still process but you may see a missmatch in frontend.")
            self.HASLOGGED_INITWARN = True



# from ...models.chargecontroller_states import ChargeControllerStates
# from .chargecontrollerbase import ChargeControllerBase


# class ChargeControllerLite(ChargeControllerBase):
#     def __init__(self, hub):
#         super().__init__(hub)

#     def _get_status_charging(self) -> ChargeControllerStates:
#         if self._hub.totalhourlyenergy.value >= self._hub.current_peak_dynamic and self._hub.locale.data.free_charge(self._hub.locale.data) is False:
#             ret = ChargeControllerStates.Stop
#         else:
#             ret = ChargeControllerStates.Start
#         return ret

#     def _get_status_connected(self, charger_state = None) -> ChargeControllerStates:
#         if charger_state is not None and self._hub.sensors.carpowersensor.value < 1 and self._is_done(charger_state):
#             ret = ChargeControllerStates.Done
#         else:
#             if (self._hub.totalhourlyenergy.value < self._hub.current_peak_dynamic) or self._hub.locale.data.free_charge(self._hub.locale.data) is True:
#                 ret = ChargeControllerStates.Start
#             else:
#                 ret = ChargeControllerStates.Stop
#         return ret
